// @(#)QuelInfo.h	1.20	10/29/91

#ifndef QuelInfo_First

#ifdef __GNUG__
#pragma once
#endif

#define QuelInfo_First

#define BOOL_TYPE	16
#define	POINT_TYPE	600
#define	LSEG_TYPE	601
#define	PATH_TYPE	602
#define	BOX_TYPE	603

#include "ByteArray.h"
#include "Geo.h"

void EscSpecial(char *in, char *out, int len= -1);
void SetDb(char *db);
void MenuEnableAll(class Menu *m);

const int	cIdFirstProc= cIdFirstUser + 1000;
const int	cIdFirstType= cIdFirstUser + 2000;
const int	cIdFirstOperator= cIdFirstUser + 3000;
const int	cIdSavePredef= cIdFirstUser + 4001;
const int	cIdFirstPredef= cIdFirstUser + 4010;
const int	cIdFirstTable= cIdFirstUser + 5000;
const int	cIdFirstAttr= 100;


class PredefInfo: public ByteArray {
  int			type;
  char			*tree;
public:
  PredefInfo(char *s, int t, char *tree);
  ~PredefInfo();

  int GetType() { return type; }
  char *GetTree() { return tree; }
};

class AttrInfo: public ByteArray {
  int	type;
  int	number;
public:
  AttrInfo(char *s, int t, int nr): type(t), number(nr), (s) {}

  int GetType() { return type; }
  int GetNumber() { return number; }
};

class ProcInfo: public ByteArray {
  int	nargs;
  int	rettype;
public:
  ProcInfo(char *s, int na, int rt): nargs(na), rettype(rt), (s) {}

  int GetRetType() { return rettype; }
  int GetNrArgs() { return nargs; }
};

class DynInfo: public ByteArray {
  char	*dynattr;
  char	*dynfunc;
  char	*dynfile;
  char	*dynbbox;
  char	*dynoid;

  char  *CheckNone(char *) const;
public:
  DynInfo(char *r, char *attr, char *bboxattr, char *func,
	char *file, char *the_oid);
  ~DynInfo();

  char * Attr() const { return dynattr; }
  char * Func() const { return dynfunc; }
  char * File() const { return dynfile; }
  char * Relation() { return Str(); }
  char * BBoxAttr() const { return CheckNone(dynbbox); }
  char * TheOid() const { return CheckNone(dynoid); }
};

class BitmapInfo: public Object {
  int		nr;
  int		usex, usey, physx, physy;
  class Bitmap	*bm;
  class ImageItem *imitem;
  short		*bw;
public:
  MetaDef(BitmapInfo);
  
  BitmapInfo(int bnr, int ux, int uy, int px, int py, char *hw, bool ishex);
  ~BitmapInfo();

  Bitmap * GetBitmap() const { return bm; }
  ImageItem * GetImageItem() const { return imitem; }
  int GetNr() const { return nr; }
};

class OprInfo: public ByteArray {
  int	nargs;
  int	rettype;
  int	argl;
  int	argr;
  char	kind;
  char	*oprcode;
public:
  OprInfo(char *s, char k, int rt, int al= 0, int ar= 0, char *code= 0);
  ~OprInfo() { SafeDelete(oprcode); }

  int GetRetType() { return rettype; }
  int GetNrArgs() { return nargs; }
  int GetArgL() { return argl; }
  int GetArgR() { return argr; }
  char GetKind() { return kind; }
  int FirstChildType();
  char *Oprcode() { return oprcode; }
};


typedef class QueryShape	*(*pqsfcp)(class MapView *, int, char **);

void *GetFuncPtr(char *filen, char *funcn);

class QuelInfo: public Object {
  char			*db_name;

  class Dictionary	*tables;
  class Dictionary	*nr2type;
  class Dictionary	*type2nr;
  class Dictionary	*regexps;
  class Dictionary	*bitmaps;
  class Dictionary	*dyns;
  class Dictionary	*dyn_dict;

  class OrdCollection	*tableoc;
  class OrdCollection	*types;
  class OrdCollection	*procs;
  class OrdCollection	*oprs;
  class OrdCollection	*predefs;
  class OrdCollection	*bitmapoc;
  class OrdCollection	*dynmenu;

  int			nproc;
  int			nopr;
  int			ntypes;
  int			nbitmap;

  class OrdCollection *GetCol(char *table);
  void FillTables();
  void FillProcs();
  void FillOprs();
  void FillTypes();
  void FillRegexps();
  void FillPredefs();
  void FillBitmaps();
  void FillDyns();
  void FillMenu();
public:
  MetaDef(QuelInfo);

  QuelInfo(char *db);
  ~QuelInfo();

  class Iterator *GetTableIter();
  class Iterator *GetAttrIter(char *table);
  bool HasAttr(char *table, char *attr, char *types= 0);
  int GetNrAttr(char *table);
  bool TableExists(char *table);

  class Iterator *GetProcIter();
  class Menu *GetProcMenu(int type);
  int GetNrProc() { return nproc; }

  class Iterator *GetTypeIter();
  char *GetTypeStr(int typeoid);
  int GetTypeOid(char *);
  class Menu *GetTypesMenu();
  int GetNrTypes() { return ntypes; }

  class Iterator *GetMenuIter();

  class Iterator *GetOprIter();
  class Menu *GetOprMenu(int type, int childtype);
  void SetOprMenu(Menu *m, int type);
  int GetNrOprs() { return nopr; }
  OprInfo *GetOprInfo(int i);
  
  class Iterator *GetDynIter();
  DynInfo *GetDynInfo(char *relation);
  pqsfcp GetDynFunc(ByteArray *is, DynInfo *di);
  
  class Iterator *GetBitmapIter();
  BitmapInfo *GetBitmapInfo(int nr);
  int GetNrBitmaps() { return nbitmap; }

  class Menu *GetPredefMenu();
  void SetPredefMenu(Menu *m, int type);
  class Iterator *GetPredefIter();
  void SavePredef(char *name, int type, Object *tr);
  Object * LoadPredef(int id);

  class Menu *GetTablesAttrMenu(bool hide_postgres= FALSE);
  class Menu *GetAttrMenu(char *table);
  AttrInfo *GetAttrInfo(int id, char **tabname);
  void SetTablesAttrMenu(class Menu *m, int type, bool is_leaf);
  void SetAttrMenu(class Menu *m, char *table, int type, bool is_leaf);

  void FillAll();
  bool TypeMatchRE(char *s, int t, char *tab= 0, char *attr= 0);
  bool TypeMatchRE(char *s, char *ts, char *tab= 0, char *attr= 0)
    { return TypeMatchRE(s, GetTypeOid(ts), tab, attr); }
};

QuelInfo *GetInfo(bool cur_db= FALSE);
QuelInfo *GetInfo(char *db);

#endif QuelInfo_First
